@@ -1,5 +1,5 @@
 # isort:skip_file
-__version__ = "0.1.2"
+__version__ = "0.2.0"
 
 
 from treeo.api import (
@@ -29,10 +29,11 @@ from treeo.mixins import (
     KindMixin,
     Immutable,
     ImmutableTree,
+    MutabilityError,
 )
 from treeo.tree import FlattenMode, FieldInfo, TreeMeta, Tree, copy, apply, make_mutable
 from treeo.types import FieldMetadata, Nothing, NOTHING, Missing, MISSING, Hashable
-from treeo.utils import OpaquePredicate, ArrayLike, Opaque, field, node, static
+from treeo.utils import ArrayLike, field, node, static
 
 
 __all__ = [
@@ -55,8 +56,6 @@ __all__ = [
     "Missing",
     "NOTHING",
     "Nothing",
-    "Opaque",
-    "OpaquePredicate",
     "Repr",
     "ToDict",
     "ToString",
@@ -79,4 +78,5 @@ __all__ = [
     "to_dict",
     "to_string",
     "make_mutable",
+    "MutabilityError",
 ]
